
# q-expansion of newform 294.4.e.d, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 294
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[197, [1, 0]], [199, [0, -1]]]
aps_data = [[0, -2], [3, -3], [0, 2], [0, 0], [8, -8], [42, 0], [-2, 2], [0, -124], [0, -76], [254, 0], [-72, 72], [0, -398], [-462, 0], [212, 0], [0, -264], [162, -162], [-772, 772], [0, 30], [764, -764], [-236, 0], [418, -418], [0, -552], [-1036, 0], [0, 30], [1190, 0], [1370, -1370], [0, 464], [0, 2136], [1226, -1226], [338, 0], [2088, 0], [0, -292], [-818, 818], [2156, 0], [0, 2850], [-1672, 1672], [446, -446], [0, -2708], [-896, 0], [0, 4034], [3480, -3480], [2898, 0], [0, -2652], [-146, 146], [-2546, 0], [-2536, 2536], [-1300, 0], [-2576, 0], [-1836, 1836], [0, -1874], [0, -3730], [2004, 0], [-646, 646], [-1260, 0], [0, 5910], [-2988, 2988], [-1318, 1318], [0, -5640], [-6446, 6446], [4930, 0], [-6260, 6260], [2310, 0], [-196, 0], [-6736, 6736], [0, 394], [0, 6714], [0, -692], [-1566, 0], [5328, -5328], [-11326, 0], [-2130, 2130], [0, -3044], [12416, -12416], [0, 7442], [100, 0], [0, 8080], [5482, -5482], [0, 10446], [0, 11334], [8594, -8594], [-10500, 0], [-12066, 0], [-4332, 4332], [1918, 0], [0, -7992], [0, -3184], [11426, 0], [0, 16934], [17038, 0], [-13592, 0], [0, 8612], [7432, -7432], [6616, -6616], [17040, 0], [0, 2948], [-17304, 0], [0, 4650], [16854, -16854], [0, -124], [0, -5382], [17460, 0], [9514, -9514], [3988, -3988], [0, -11346], [8436, -8436], [2098, -2098], [-9436, 0], [0, -1314], [8940, -8940], [-16058, 0], [0, 3936], [-174, 174], [16018, 0], [-3068, 3068], [24656, 0], [-7594, 7594], [3724, 0], [3792, -3792], [0, -24702], [-20144, 0], [-2522, 2522], [-10414, 0], [0, -22230], [-18192, 18192], [0, 8108], [-5794, 0], [0, 1954], [0, -32016], [23072, 0], [0, 31782], [24396, -24396], [-32604, 0], [0, 7680], [366, 0], [0, 29374], [38990, 0], [-20470, 20470], [29916, -29916], [-4914, 0], [-34250, 34250], [41804, 0], [0, -30862], [-10576, 10576], [-10680, 0], [-1178, 1178], [5600, 0], [826, 0], [45918, -45918], [0, 42380], [0, 26524], [0, -20614], [23730, 0], [-9028, 0], [0, 37200], [-23988, 23988], [15276, 0], [0, 10760], [0, -52890], [6118, 0], [-32230, 32230], [0, 18544], [25930, 0], [8192, 0], [0, -54444], [25446, -25446], [33192, -33192], [-11024, 11024], [-40714, 40714]]
